// ------|---------|---------|---------|---------|---------|---------|-------80|---------|------100|
/**
 * @file: AClass.h
 * @author rand huso
 * @version 1.0
 *
 * created: 12 September 2003
 * purpose: to test non-type parameter usage
 */
// ------|---------|---------|---------|---------|---------|---------|-------80|---------|------100|

#ifndef _ACLASSH_
#define _ACLASSH_

#include <iostream.h>
#include <string.h>
#include <stdio.h>


class AClass {

    public:

		/**
         * Constructor for simple object - initialized to zero.
		 */

        AClass() {
            m_Count         = ++m_StaticCount;
            m_Value[ 0 ]    = (char)0;
        }

		/**
         * Constructor from a string containing a numeric value.
		 */

        AClass( const char * label ) {
            m_Count    = ++m_StaticCount;
            float labelValue;
            sscanf( label, "%f", &labelValue );
            sprintf( m_Value, "%f", labelValue );
        }

		/**
         * Destructor - which does not need to remove allocated memory
		 */

        virtual ~AClass() {
            m_Value[ 0 ] = (char)0;
        }

		/**
         * Constructor from existing object (copy constructor)
		 */

        AClass( const AClass & right ) {
            strcpy( m_Value, right.m_Value );
            m_Count = ++m_StaticCount;
        }

		/**
         * Assignment operator.
		 *
		 * @param right source value
		 * @return the current object filled out by the right
		 */

        AClass & operator = ( const AClass & right ) {
            if( &right == this ) return *this;
            strcpy( m_Value, right.m_Value );
            return *this;
        }

		/**
         * Stream output function returns the simple stored object enclosed in brackets ("<" and ">").
		 */

        friend ostream & operator << ( ostream & os, const AClass & right ) {
            float currentLeft;
            sscanf( right.m_Value, "%f", &currentLeft );
			os.precision( 2 );
			os << "<" << currentLeft << ">";
            return os;
        }

        // --------|---------|---------|---------|---------|---------|-------80|---------|------100|
        // binary operators multiplication, division, addition, subtraction
        // --------|---------|---------|---------|---------|---------|-------80|---------|------100|

		/**
         * In place addition operator.
		 *
		 * @param right is the object to add to this one
		 * @return the object with the addition of the input value
		 */

        AClass & operator += ( const AClass & right ) {
            float currentLeft;
            sscanf( m_Value, "%f", &currentLeft );
            float currentRight;
            sscanf( right.m_Value, "%f", &currentRight );
            sprintf( m_Value, "%f", ( currentLeft+currentRight ));
            return *this;
        }

		/**
         * In place subtraction operator.
		 *
		 * @param right is the object to subtract from this one
		 * @return the object less the input value
		 */

        AClass & operator -= ( const AClass & right ) {
            float currentLeft;
            sscanf( m_Value, "%f", &currentLeft );
            float currentRight;
            sscanf( right.m_Value, "%f", &currentRight );
            sprintf( m_Value, "%f", ( currentLeft-currentRight ));
            return *this;
        }

		/**
         * In place multiplication operator.
		 *
		 * @param right is the object to multiply times this one
		 * @return the object times the input value
		 */

        AClass & operator *= ( const AClass & right ) {
            float currentLeft;
            sscanf( m_Value, "%f", &currentLeft );
            float currentRight;
            sscanf( right.m_Value, "%f", &currentRight );
            sprintf( m_Value, "%f", ( currentLeft*currentRight ));
            return *this;
        }

		/**
         * In place division operator.
		 *
		 * @param right is the object to divide into this one
		 * @return the object divided by the input value
		 */

        AClass & operator /= ( const AClass & right ) {
            float currentLeft;
            sscanf( m_Value, "%f", &currentLeft );
            float currentRight;
            sscanf( right.m_Value, "%f", &currentRight );
            sprintf( m_Value, "%f", ( currentLeft/currentRight ));
            return *this;
        }

		/**
         * Unary negation.
		 *
		 * A copy of the original object is made, the value
		 * is set to a negative number.
		 *
		 * @return the negative value from the original object
		 */

        AClass operator - () {
            AClass answer( *this );
            float currentLeft;
            sscanf( m_Value, "%f", &currentLeft );
            sprintf( m_Value, "%f", ( -currentLeft ));
            return answer;
        }

		/**
         * Postfix increment operator.
		 *
		 * The presence of the dummy argument indicates a postfix operator, as opposed to
		 * a prefix operator. This routine calls the prefix operator after making a copy
		 * of the original to return.
		 *
		 * @return an unmodified copy of the original, which is now incremented
		 */

        AClass operator ++ ( int ) {
            AClass temp( *this );
            operator ++ ();
            return temp;
        }

		/**
         * Prefix increment operator.
		 *
		 * @return the incremented value
		 */

        AClass & operator ++ () {
            float currentLeft;
            sscanf( m_Value, "%f", &currentLeft );
            sprintf( m_Value, "%f", ( ++currentLeft ));
            return *this;
        }

		/**
		 * Logical equivalence operator. Compares the contents of the
		 * stored value and returns true if the content is equivalent.
		 *
         * @return boolean comparison result using string comparison
		 */

        bool operator == ( const AClass & right ) const {
            return ( 0 == strcmp( m_Value, right.m_Value ));
        }

		/**
		 * Logical inequivalence operator. Compares the contents of the
		 * stored value and returns false if the content is equivalent.
		 *
         * @return negated boolean comparison from operator==
		 */

        bool operator != ( const AClass & right ) const {
            return !( *this == right );
        }

    private:
        static  int     m_StaticCount;    /** count of the number of AClass instances */
                int     m_Count;          /** instance number */
                char    m_Value[ 10 ];    /** numeric value storage */
};

int        AClass::m_StaticCount    = 0;

#endif

// ------|---------|---------|------< EOF >------|---------|---------|-------80|---------|------100|
