// ------|---------|---------|---------|---------|---------|---------|-------80|---------|------100|
/**
 * @file test.cpp
 * @author rand huso
 * @version 1.0
 *
 * created 12 September 2003
 * purpose to drive the templates RVector and RMatrix
 */
// ------|---------|---------|---------|---------|---------|---------|-------80|---------|------100|

#include <iostream.h>
#include "RMatrix.h"
#include "RVector.h"
#include "AClass.h"

#define USE_ACLASS 1
#if USE_ACLASS

/** the AClass equivalent of zero */
AClass Empty( "0" );
/** the AClass equivalent of one */
AClass Unity( "1" );
/** the AClass equivalent of two */
AClass Two( "2" );
/** the AClass equivalent of three */
AClass Three( "3" );

typedef RVector< AClass, &Empty > RVc;
typedef RMatrix< AClass, &Empty, &Unity > RMa;

#else

/** the float value of zero */
float Empty = 0.0;
/** the float value of one */
float Unity = 1.0;
/** the float value of two */
float Two   = 2.0;
/** the float value of three */
float Three = 3.0;

typedef RVector< float, &Empty > RVc;
typedef RMatrix< float, &Empty, &Unity > RMa;

#endif

/**
 * Drives the testing of the RMatrix and RVector templates.
 *
 */

int main( int argC, char *argV[] ) {
    // --|---------|---------|---------|---------|---------|---------|-------80|---------|------100|
    cerr << endl << endl << "Starting" << endl;

#if USE_ACLASS
	cerr << "Using the AClass class for this test." << endl;
#else
	cerr << "Using the intrinsic float type for this test." << endl;
#endif

    // --|---------|---------|---------|---------|---------|---------|-------80|---------|------100|
    // Tests for the RVector class
    cerr << endl << "Tests for RVector." << endl;
    cerr << "Constructors." << endl;
    RVc vector1;
    cerr << "[vector1:" << vector1 << "]" << endl;
    RVc vector2( 4 );
    cerr << "[vector2:" << vector2 << "]" << endl;
    RVc vector3( vector2 );
    cerr << "[vector3:" << vector3 << "]" << endl;

    cerr << "Assignment, length, and indexing." << endl;
    cerr << "[vector1=vector2:" << ( vector1=vector2 ) << "]" << endl;
    cerr << "[vector3.Length():" << vector3.Length() << "]" << endl;
    cerr << "[vector1[1]=Three" << ( vector1[1]=Three ) << "]" << endl;

    // --|---------|---------|---------|---------|---------|---------|-------80|---------|------100|
    cerr << endl << "Operators (part 1)." << endl;
    cerr << "[vector1:" << vector1 << "]" << endl;
    cerr << "[vector1+vector2:" << ( vector1+vector2 ) << "]" << endl;
    cerr << "[vector1:" << vector1 << "]" << endl;
    cerr << "[vector1+=vector2:" << ( vector1+=vector2 ) << "]" << endl;
    cerr << "[vector1:" << vector1 << "]" << endl;

    vector1 = RVc( 3, Three );
    cerr << "[vector1:" << vector1 << "]" << endl;
    vector2 = RVc( 2, Unity );
    cerr << "[vector2:" << vector2 << "]" << endl;

    cerr << "[vector1-vector2:" << ( vector1-vector2 ) << "]" << endl;
    cerr << "[vector1:" << vector1 << "]" << endl;
    cerr << "[vector1-=vector2:" << ( vector1-=vector2 ) << "]" << endl;
    cerr << "[vector1:" << vector1 << "]" << endl;

    vector1 = RVc( 3, Three );
    cerr << "[vector1:" << vector1 << "]" << endl;
    vector2 = RVc( 2, Unity );
    cerr << "[vector2:" << vector2 << "]" << endl;

    cerr << "[vector1/vector2:" << ( vector1/vector2 ) << "]" << endl;
    cerr << "[vector1:" << vector1 << "]" << endl;
    cerr << "[vector1/=vector2:" << ( vector1/=vector2 ) << "]" << endl;
    cerr << "[vector1:" << vector1 << "]" << endl;

    vector1 = RVc( 3, Three );
    cerr << "[vector1:" << vector1 << "]" << endl;
    vector2 = RVc( 2, Unity );
    cerr << "[vector2:" << vector2 << "]" << endl;

    cerr << "[vector1&vector2:" << ( vector1&vector2 ) << "]" << endl;
    cerr << "[vector1:" << vector1 << "]" << endl;
    cerr << "[vector1&=vector2:" << ( vector1&=vector2 ) << "]" << endl;
    cerr << "[vector1:" << vector1 << "]" << endl;

    // --|---------|---------|---------|---------|---------|---------|-------80|---------|------100|
    cerr << endl << "Operators (part 2)." << endl;
    vector1 = RVc( 3, Three );
    cerr << "[vector1:" << vector1 << "]" << endl;
    vector2 = RVc( 4, Unity );
    cerr << "[vector2:" << vector2 << "]" << endl;

    cerr << "[vector1*vector2:" << ( vector1*vector2 ) << "]" << endl;
    cerr << "[vector1*Unity:" << ( vector1*Unity ) << "]" << endl;
    cerr << "[vector1:" << vector1 << "]" << endl;
    cerr << "[Unity*vector1:" << ( Unity*vector1 ) << "]" << endl;
    cerr << "[vector1:" << vector1 << "]" << endl;
    cerr << "[vector1*=Unity:" << ( vector1*=Unity ) << "]" << endl;
    cerr << "[vector1:" << vector1 << "]" << endl;

    cerr << "[vector1/Unity:" << ( vector1/Unity ) << "]" << endl;
    cerr << "[vector1:" << vector1 << "]" << endl;
    cerr << "[vector1/=Unity:" << ( vector1/=Unity ) << "]" << endl;
    cerr << "[vector1:" << vector1 << "]" << endl;

    cerr << "[-vector1:" << ( -vector1 ) << "]" << endl;
    cerr << "[vector1:" << vector1 << "]" << endl;

    vector1 = RVc( 3, Three );
    cerr << endl << "[vector1:" << vector1 << "]" << endl;
    vector2 = RVc( 3, Three );
    cerr << "[vector2:" << vector2 << "]" << endl;

    cerr << "[vector1==vector2:" << ( vector1==vector2 ) << "]" << endl;
    cerr << "[vector1!=vector2:" << ( vector1!=vector2 ) << "]" << endl;

    vector1 = RVc( 3, Three );
    cerr << "[vector1:" << vector1 << "]" << endl;
    vector2 = RVc( 4, Unity );
    cerr << "[vector2:" << vector2 << "]" << endl;

    cerr << "[vector1==vector2:" << ( vector1==vector2 ) << "]" << endl;
    cerr << "[vector1!=vector2:" << ( vector1!=vector2 ) << "]" << endl;

    vector2 = RVc( 4, Empty );
    vector2[ 1 ] = Unity;
    vector2[ 2 ] = Three;
    cerr << "[vector2:" << vector2 << "]" << endl;
    vector2.Swap( 1, 2 );
    cerr << "[vector2.Swap(1,2):" << vector2 << "]" << endl;

    cerr << endl << endl;

    // --|---------|---------|---------|---------|---------|---------|-------80|---------|------100|
    // Tests for the RMatrix class
    cerr << "Tests for RMatrix." << endl;
    cerr << "Constructors." << endl;
    RMa matrix1;
    cerr << "[matrix1:" << matrix1 << "]" << endl;
    RMa matrix2( 3, 3 );
    cerr << "[matrix2:" << matrix2 << "]" << endl;
    RMa matrix3 = RMa( matrix2 );
    cerr << "[matrix3:" << matrix3 << "]" << endl;

    cerr << endl << endl;

    // --|---------|---------|---------|---------|---------|---------|-------80|---------|------100|
    cerr << "Operators (part 1)." << endl;
    matrix1 = RMa( 2, 4, Unity );
    cerr << "[matrix1:" << matrix1 << "]" << endl;
    matrix2 = RMa( 3, 3, Three );
    cerr << "[matrix2:" << matrix2 << "]" << endl;

    cerr << "[matrix1+matrix2:" << ( matrix1+matrix2 ) << "]" << endl;
    cerr << "[matrix2+matrix1:" << ( matrix2+matrix1 ) << "]" << endl;
    cerr << "[matrix1+=matrix2:" << ( matrix1+=matrix2 ) << "]" << endl;
    cerr << "[matrix1:" << matrix1 << "]" << endl;

    matrix1 = RMa( 2, 4, Unity );
    cerr << "[matrix1:" << matrix1 << "]" << endl;

    cerr << "[matrix1-matrix2:" << ( matrix1-matrix2 ) << "]" << endl;
    cerr << "[matrix1-=matrix2:" << ( matrix1-=matrix2 ) << "]" << endl;
    cerr << "[matrix1:" << matrix1 << "]" << endl;

    matrix1 = RMa( 2, 4, Unity );
    cerr << "[matrix1:" << matrix1 << "]" << endl;

    cerr << "[matrix1&matrix2:" << ( matrix1&matrix2 ) << "]" << endl;
    cerr << "[matrix1&=matrix2:" << ( matrix1&=matrix2 ) << "]" << endl;
    cerr << "[matrix1:" << matrix1 << "]" << endl;

    matrix1 = RMa( 2, 4, Unity );
    cerr << "[matrix1:" << matrix1 << "]" << endl;

    cerr << "[matrix1*Three:" << ( matrix1*Three ) << "]" << endl;
    cerr << "[Three*matrix1:" << ( Three*matrix1 ) << "]" << endl;
    cerr << "[matrix1*=Three:" << ( matrix1*=Three ) << "]" << endl;

    cerr << "[matrix1/Three:" << ( matrix1/Three ) << "]" << endl;
    cerr << "[matrix1/=Three:" << ( matrix1/=Three ) << "]" << endl;

    matrix1 = RMa( 2, 4, Unity );
    cerr << "[matrix1:" << matrix1 << "]" << endl;

    cerr << "[-matrix1:" << -matrix1 << "]" << endl;

    cerr << endl << endl;

    // --|---------|---------|---------|---------|---------|---------|-------80|---------|------100|
    cerr << "Operators (part 2)." << endl;
    cerr << "[matrix1:" << matrix1 << "]" << endl;
    RMa::RV vectorA( 3, Three );
    cerr << "[vectorA:" << vectorA << "]" << endl;
    cerr << "[matrix1*vectorA:" << matrix1*vectorA << "]" << endl;

    matrix1 = RMa( 3, 2, Three );
    cerr << "[matrix1:" << matrix1 << "]" << endl;
    matrix2 = RMa( 3, 2, Three );
    cerr << "[matrix2:" << matrix2 << "]" << endl;
    cerr << "[matrix1==Matrix2:" << ( matrix1==matrix2 ) << "]" << endl;
    cerr << "[matrix1!=Matrix2:" << ( matrix1!=matrix2 ) << "]" << endl;

    cerr << endl << endl;

    // --|---------|---------|---------|---------|---------|---------|-------80|---------|------100|
    cerr << "Operators (part 3 - Bandwidth Reduction, multiplication, inversion)." << endl;

    cerr << "[Bandwidth Reduction]" << endl;
    matrix1 = RMa( 7, 7 ); ///minimum of 4,4 - used below

    matrix1[ 0 ][ 0 ] = Unity;
    matrix1[ 1 ][ 0 ] = Unity;
    matrix1[ 1 ][ 1 ] = Unity;
    matrix1[ 2 ][ 0 ] = Unity;
    matrix1[ 2 ][ 1 ] = Unity;
    matrix1[ 3 ][ 2 ] = Unity;
    matrix1[ 4 ][ 2 ] = Unity;
    matrix1[ 4 ][ 3 ] = Unity;
    matrix1[ 4 ][ 4 ] = Unity;
    matrix1[ 5 ][ 3 ] = Unity;
    matrix1[ 5 ][ 4 ] = Unity;
    matrix1[ 6 ][ 0 ] = Unity;
    matrix1[ 6 ][ 1 ] = Unity;
    matrix1[ 6 ][ 5 ] = Unity;
    matrix1[ 6 ][ 6 ] = Unity;

    // the matrix for bandwidth reduction must be symmetric
    for( int row=0; row<matrix1.rows(); ++row ) {
        for( int col=0; col<row; ++col ) {
            matrix1[ col ][ row ] = matrix1[ row ][ col ];
        }
    }
    cerr << "[matrix1:" << matrix1 << "]" << endl;
    cerr << "[matrix1.Reduce():" << ( matrix1.Reduce()) << "]" << endl;

    cerr << endl << endl;

    // --|---------|---------|---------|---------|---------|---------|-------80|---------|------100|
    cerr << "[matrix1.Identity():" << ( matrix1.Identity()) << "]" << endl;

    cerr << endl << endl;

    cerr << "[Multiplication]" << endl;
    matrix1[ 0 ][ 0 ] = Unity;
    matrix1[ 1 ][ 1 ] = Unity;
    matrix1[ 2 ][ 2 ] = Unity;
    matrix1[ 3 ][ 3 ] = Unity;

    matrix1[ 2 ][ 0 ] = Unity;
    matrix1[ 0 ][ 2 ] = Unity;
    matrix1[ 3 ][ 1 ] = Unity;
    matrix1[ 1 ][ 3 ] = Unity;

    cerr << "[matrix1:" << matrix1 << "]" << endl;
    cerr << "[matrix2:" << matrix2 << "]" << endl;
    cerr << "[matrix1*Matrix2:" << ( matrix1*matrix2 ) << "]" << endl;

    cerr << endl << endl;

    // --|---------|---------|---------|---------|---------|---------|-------80|---------|------100|
    cerr << "[Inversion]" << endl;
    matrix1 = RMa( 3, 4 );
    matrix1[ 0 ][ 1 ] = Two;
    matrix1[ 0 ][ 3 ] = Two;
    matrix1[ 1 ][ 0 ] = Two;
    matrix1[ 1 ][ 1 ] = Two;
    matrix1[ 1 ][ 3 ] = Two;
    matrix1[ 2 ][ 0 ] = Two;
    matrix1[ 2 ][ 2 ] = Two;
    cerr << "[matrix1:" << matrix1 << "]" << endl;

    cerr << "[~matrix1:" << ~matrix1 << "]" << endl;

    cerr << endl << endl;

    matrix1 = RMa( 2, 3 );
    matrix1[ 0 ][ 1 ] = Two;
    matrix1[ 1 ][ 0 ] = Unity;
    matrix1[ 1 ][ 2 ] = Two;
    cerr << "[matrix1:" << matrix1 << "]" << endl;

    cerr << endl << endl;

    matrix2 = ~matrix1;
    cerr << "[matrix2 (inverse of matrix1:" << matrix2 << "]" << endl;

    cerr << endl << endl;

    // --|---------|---------|---------|---------|---------|---------|-------80|---------|------100|
    // matrix for general solution
    cerr << "[ImA12A:" << matrix1.ImA12A( matrix2 ) << "]" << endl;

    cerr << "Done" << endl << endl << endl;
    return 0;
}

// ------|---------|---------|------< EOF >------|---------|---------|-------80|---------|------100|
